@@ -21,6 +21,8 @@ blav_model_test <- function(lavmodel       = NULL,
     if("syntax" %in% names(jagextra)){
         warning("blavaan WARNING: Marginal log-likelihood cannot be approximated when there is additional JAGS syntax.", call. = FALSE)
         mll <- NA
+    } else if(lavoptions$categorical){
+        mll <- NA # not tested, priors may cause problems
     } else {
         mll <- try(margloglik(lavpartable, lavmodel, lavoptions, 
                               lavsamplestats, lavdata, lavcache,
@@ -29,10 +31,14 @@ blav_model_test <- function(lavmodel       = NULL,
         if(inherits(mll, "try-error")) mll <- NA
     }
 
-    ppp <- postpred(lavpartable, lavmodel, lavoptions,
-                    lavsamplestats, lavdata, lavcache, lavjags,
-                    samplls, lavobject)$ppval
-
+    if(lavoptions$target == "stan") {
+        ppp <- stansumm['ppp', 'mean']
+    } else {
+        ppp <- postpred(lavpartable, lavmodel, lavoptions,
+                        lavsamplestats, lavdata, lavcache, lavjags,
+                        samplls, lavobject)$ppval
+    }
+        
     TEST[[1]] <- list(test="mloglik",
                       stat=as.numeric(mll),
                       stat.group=as.numeric(NA),
