@@ -2,7 +2,7 @@
     version <- read.dcf(file=system.file("DESCRIPTION", package=pkgname),
                       fields="Version")
     packageStartupMessage("This is ",paste(pkgname, version))
-    packageStartupMessage('On multicore systems, we suggest use of future::plan("multiprocess") for faster post-MCMC computations.')
+    packageStartupMessage('On multicore systems, we suggest use of future::plan("multicore") or\n', '  future::plan("multisession") for faster post-MCMC computations.')
 }
 
 .onLoad <- function(libname, pkgname) {
